<?php

namespace Drupal\omni_twig\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\LanguageInterface;
use Drupal\Core\Field\FieldItemListInterface;

/**
 * Plugin implementation of the 'omni_twig' formatter.
 *
 * @FieldFormatter(
 *   id = "omni_twig",
 *   label = @Translation("Twig Template"),
 *   field_types = {}
 * )
 */
class OmniTwig extends FormatterBase {

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    return [$this->t('Render the field using a Twig Template.')];
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'template' => "{# This will render string or text fields properly. Modify this for other field types. #}\n{% set class_name = (name ~ '--bundle-' ~ bundle)|clean_class %}\n<div class=\"{{ class_name }}\">\n  {% for item in items %}\n    {{ item.value|raw }}\n  {% endfor %}\n</div>",
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    return [
      'template' => [
        '#title' => $this->t('Twig Template'),
        '#type' => 'textarea',
        '#default_value' => $this->getSetting('template'),
        '#description' => 'Additional Features:<ul><li>Variables: items [all field items], field [the first field item], entity [of this field], parent [entity], bundle, name, label, label_display, item_list [FieldItemListInterface], field_definition.</li><li>Functions: dump(variable).</li><li>Filters: formatter(name, settings).</li></ul>',
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function view(FieldItemListInterface $items, $langcode = NULL) {
    // Default the language to the current content language.
    if (empty($langcode)) {
      $langcode = \Drupal::languageManager()->getCurrentLanguage(LanguageInterface::TYPE_CONTENT)->getId();
    }

    $definition = $items->getFieldDefinition();
    $entity = $items->getEntity();
    $elements = [
      '#type' => 'inline_template',
      '#template' => $this->getSetting('template'),
      '#context' => [
        'langcode' => $langcode,
        'label_display' => $this->label,
        'label' => $definition->getLabel(),
        'parent' => $items->getParent()->getEntity(),
        'entity' => $entity,
        'bundle' => $entity->bundle(),
        'name' => $items->getName(),
        'field_definition' => $definition,
        'item_list' => $items,
        'items' => $items->getIterator(),
        'field' => $items->get(0),
      ],
    ];

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {}

}
